﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Configuration;

namespace SerialPortConsole
{
    class Program
    {
        static string port = string.Empty;
        static int baud = 0;
        static SerialPort sp;

        static void Main(string[] args)
        {
            Boolean portSelected = false;
            Boolean baudSelected = false;
            

            //
            //List and select COM ports
            //
            string[] ports = ListSerialPorts();
            while (!portSelected)
            {
                int portIdx = GetPort();

                if (portIdx >= 0 & ports.Count() > portIdx)
                {
                    port = ports[portIdx];
                    Console.WriteLine("\nYou selected {0}\n", port);
                    portSelected = true;
                }
                else
                {
                    Console.WriteLine("\nPort Id can not be greater than {0}\nTry again!\n\n ", ports.Count() - 1);
                }
            }

            while (!baudSelected)
            {
                baudSelected = GetBaud();
            }

            Console.WriteLine("\n---------------------------------------\n");
            //Init the serial port with user selected data
            sp = new SerialPort(port, baud);
            sp.Open();

            //assign a serial port handler
            sp.DataReceived += new SerialDataReceivedEventHandler(SerialPortDataReceived);

            // Receive bytes forever
            while (true) { }

        }

        static void SerialPortDataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                byte[] buff = new byte[sp.BytesToRead];
                int result = sp.Read(buff, 0, sp.BytesToRead);
                /*
                foreach (byte b in buff)
                    Console.Write(Encoding.ASCII.GetString(buff));
                */
                foreach (byte b in buff)
                    Console.Write("{0}", b.ToString().PadRight(3));
                
                

            }
            catch (Exception ex)
            {
                Console.WriteLine("\n{0}\n", ex.ToString());
            }
        }

        static string[] ListSerialPorts()
        {
            // Get a list of serial port names.
            string[] ports = SerialPort.GetPortNames();
            int i = 0;

            Console.WriteLine("Availalbe Serial Ports {0}\n", ports.Count());

            // Display each port name to the console.
            foreach (string port in ports)
                Console.WriteLine("{0}: {1}", i.ToString().PadRight(2), port);

            Console.WriteLine();

            return ports;
        }

        static int GetPort()
        {
            Console.Write("Enter a port ID and press enter: ");
            string portId = Console.ReadLine();

            int result = 0;
            if (int.TryParse(portId, out result))
                return result;
            else
                return -1;
         }

        static Boolean GetBaud()
        {
            Console.Write("Enter BAUD: ");
            string sbaud = Console.ReadLine();
            if (int.TryParse(sbaud, out baud))
                return true;
            else
                Console.WriteLine("\nYou must enter a number from 2400 to 1000000\n");
                return false;
        }

    }
}
