﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Configuration;


namespace ServoSlider
{
    public partial class Form1 : Form
    {
        private SerialPort sp;
        string port = "COM11";
        int baud = 115200;

        // serail port data received thread safe delegate
        public delegate void SetTextCallback(String message);

        public Form1()
        {
            InitializeComponent();
            sp = new SerialPort(port, baud);

            //assign a serial port handler
            sp.DataReceived += new SerialDataReceivedEventHandler(SerialPortDataReceived);
            try
            {
                sp.Open();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void SendCommand(byte[] buff)
        {
            sp.Write(buff, 0, buff.Count());
        }

        private byte[] SerializeCommand(byte Id, string cmd)
        {
            byte[] bytes = null;
            List<byte> buff = new List<byte>();

            //Validate user entry and conver to an integer
            int value = 0;
            if (!int.TryParse(cmd, out value))
            {
                MessageBox.Show(@"Invalid command {0}", cmd.ToString());
                return bytes;
            }
            //Convert the int to a byte array
            bytes = BitConverter.GetBytes(value);


            //Build the command
            buff.Add((byte)'!');
            buff.Add(Id);
            buff.Add((byte)bytes.Count());
            foreach (byte b in bytes)
            {
                buff.Add(b);
            }
            buff.Add(0x0D);

            return buff.ToArray();

        }

        private void tbServo1_Scroll(object sender, EventArgs e)
        {
            //Update label
            this.lblServo1.Text = this.tbServo1.Value.ToString();

            //Create and send command
            byte[] cmdbuff = SerializeCommand(1, this.tbServo1.Value.ToString());
            SendCommand(cmdbuff);

        }

        private void tbServo2_Scroll(object sender, EventArgs e)
        {
            //Update label
            this.lblServo2.Text = this.tbServo2.Value.ToString();

            //Create and send command
            byte[] cmdbuff = SerializeCommand(2, this.tbServo2.Value.ToString());
            SendCommand(cmdbuff);
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            sp.Close();
        }


        #region Set Text
        // Align the caller and control thread
        private void SetText(String message)
        {
            try
            {
                if (lblStatus.InvokeRequired)
                {
                    SetTextCallback del = new SetTextCallback(SetText);
                    this.Invoke(del, message);
                }
                else
                {
                    lblStatus.Text = message;
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        #endregion

        private void SerialPortDataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                byte[] buff = new byte[sp.BytesToRead];
                int result = sp.Read(buff, 0, sp.BytesToRead);
                SetText(Encoding.ASCII.GetString(buff));
            }
            catch (Exception ex)
            {
                Console.WriteLine("\n{0}\n", ex.ToString());
            }
        }

    }
}
